@@ -15,13 +15,34 @@ static NSString const *typeCheck = @"check";
 static NSString const *typeDisabled = @"disabled";
 static NSString const *typeSeparator = @"separator";
 static NSString const *typeSlider = @"slider";
-static NSString const *typeLightTemp = @"typeLightTemp";
+static NSString const *typeLightTemp = @"lightTemp";
+static NSString const *typeTurboSlider = @"turboSlider";
 
 @implementation GBSettingsViewController
 {
     NSArray<NSDictionary *> *_structure;
     UINavigationController *_detailsNavigation;
     NSArray<NSArray<GBTheme *> *> *_themes; // For prewarming
+    bool _iPadRoot;
+    __weak UISlider *_turboSlider;
+}
+
++ (UIImage *)settingsImageNamed:(NSString *)name
+{
+    UIImage *base = [UIImage imageNamed:name];
+    UIGraphicsBeginImageContextWithOptions(base.size, false, base.scale);
+    UIBezierPath *path = [UIBezierPath bezierPathWithRoundedRect:(CGRect){{0, 0}, base.size} cornerRadius:8];
+    CGContextSaveGState(UIGraphicsGetCurrentContext());
+    [path addClip];
+    [base drawInRect:path.bounds];
+    if (@available(iOS 19.0, *)) {
+        CGContextRestoreGState(UIGraphicsGetCurrentContext());
+        UIImage *overlay = [UIImage imageNamed:@"settingsOverlay"];
+        [overlay drawInRect:path.bounds];
+    }
+    UIImage *ret = UIGraphicsGetImageFromCurrentImageContext();
+    UIGraphicsEndImageContext();
+    return ret;
 }
 
 + (NSArray<NSDictionary *> *)rootStructure
@@ -48,17 +69,32 @@ static NSString const *typeLightTemp = @"typeLightTemp";
             ]
         },
         @{
-            @"header": @"Turbo Speed",
+            @"header": @"Turbo Speed Cap",
             @"items": @[
-                    @{@"type": typeRadio, @"pref": @"GBTurboSpeed", @"title": @"200%",     @"value": @2,},
-                    @{@"type": typeRadio, @"pref": @"GBTurboSpeed", @"title": @"400%",     @"value": @4,},
-                    @{@"type": typeRadio, @"pref": @"GBTurboSpeed", @"title": @"Uncapped", @"value": @1,},
+                @{@"type": typeCheck, @"pref": @"GBTurboCap", @"title": @"Cap Turbo Speed", @"block": ^void(GBSettingsViewController *controller) {
+                    if ([[NSUserDefaults standardUserDefaults] floatForKey:@"GBTurboCap"] == 1.0) {
+                        if (controller->_turboSlider) {
+                            [[NSUserDefaults standardUserDefaults] setFloat:controller->_turboSlider.value forKey:@"GBTurboCap"];
+                            controller->_turboSlider.enabled = true;
+                        }
+                        else {
+                            [[NSUserDefaults standardUserDefaults] setFloat:2.0 forKey:@"GBTurboCap"];
+                        }
+                    }
+                    else {
+                        controller->_turboSlider.enabled = false;
+                    }
+                }},
+                    @{@"type": typeTurboSlider, @"pref": @"GBTurboCap", @"min": @1.5, @"max": @4, @"minImage": @"tortoise.fill", @"maxImage": @"hare.fill"},
             ],
             @"footer": ^NSString *(){
                 if ([[NSUserDefaults standardUserDefaults] boolForKey:@"GBDynamicSpeed"]) {
-                    return @"This setting will have no effect because horizontal swipes are configured to dynamically control speed in the “Controls” settings";
+                    return @"This setting will have no effect because horizontal swipes are configured to dynamically control speed in the “Controls” settings.";
                 }
-                return @"";
+                if ([[NSUserDefaults standardUserDefaults] doubleForKey:@"GBTurboCap"]) {
+                    return [NSString stringWithFormat:@"Turbo speed is capped to %u%%", (unsigned)round([[NSUserDefaults standardUserDefaults] doubleForKey:@"GBTurboCap"] * 100)];
+                }
+                return @"Turbo speed is not capped";
             },
         },
         @{
@@ -169,7 +205,7 @@ static NSString const *typeLightTemp = @"typeLightTemp";
                                 @{@"type": typeRadio, @"pref": @"GBFilter", @"title": @"HQ2x", @"value": @"HQ2x"},
                                 @{@"type": typeRadio, @"pref": @"GBFilter", @"title": @"OmniScale", @"value": @"OmniScale"},
                                 @{@"type": typeRadio, @"pref": @"GBFilter", @"title": @"OmniScale Legacy", @"value": @"OmniScaleLegacy"},
-                                @{@"type": typeRadio, @"pref": @"GBFilter", @"title": @"AA OmniScale Legacy", @"value": @"AAOmniScaleLegacy"},
+                                @{@"type": typeRadio, @"pref": @"GBFilter", @"title": @"Anti-aliased OmniScale Legacy", @"value": @"AAOmniScaleLegacy"},
                             ]
                         },
                     ]
@@ -365,31 +401,31 @@ static NSString const *typeLightTemp = @"typeLightTemp";
                         @"title": @"Emulation",
                         @"type": typeSubmenu,
                         @"submenu": emulationMenu,
-                        @"image": [UIImage imageNamed:@"emulationSettings"],
+                        @"image": [self settingsImageNamed:@"emulationSettings"],
                     },
                     @{
                         @"title": @"Video",
                         @"type": typeSubmenu,
                         @"submenu": videoMenu,
-                        @"image": [UIImage imageNamed:@"videoSettings"],
+                        @"image": [self settingsImageNamed:@"videoSettings"],
                     },
                     @{
                         @"title": @"Audio",
                         @"type": typeSubmenu,
                         @"submenu": audioMenu,
-                        @"image": [UIImage imageNamed:@"audioSettings"],
+                        @"image": [self settingsImageNamed:@"audioSettings"],
                     },
                     @{
                         @"title": @"Controls",
                         @"type": typeSubmenu,
                         @"submenu": controlsMenu,
-                        @"image": [UIImage imageNamed:@"controlsSettings"],
+                        @"image": [self settingsImageNamed:@"controlsSettings"],
                     },
                     @{
                         @"title": @"Themes",
                         @"type": typeSubmenu,
                         @"class": [GBThemesViewController class],
-                        @"image": [UIImage imageNamed:@"themeSettings"],
+                        @"image": [self settingsImageNamed:@"themeSettings"],
                     },
     ];
     
@@ -416,12 +452,20 @@ static NSString const *typeLightTemp = @"typeLightTemp";
         return controller;
     }
     
-    UISplitViewController *split = [[UISplitViewController alloc] init];
+    UISplitViewController *split = nil;
+    if (@available(iOS 14.5, *)) {
+        split = [[UISplitViewController alloc] initWithStyle:UISplitViewControllerStyleDoubleColumn];
+        split.displayModeButtonVisibility = UISplitViewControllerDisplayModeButtonVisibilityNever;
+    }
+    else {
+        split = [[UISplitViewController alloc] init];
+    }
     UIViewController *blank = [[UIViewController alloc] init];
     blank.view.backgroundColor = root.view.backgroundColor;
     root->_detailsNavigation = [[UINavigationController alloc] initWithRootViewController:blank];
     split.viewControllers = @[controller, root->_detailsNavigation];
     split.preferredDisplayMode = UISplitViewControllerDisplayModeAllVisible;
+    root->_iPadRoot = true;
     return split;
 }
 
@@ -569,6 +613,8 @@ static NSString *LocalizedNameForElement(GCControllerElement *element, GBControl
                 @{@"type": typeRadio, @"getter": getter, @"setter": setter, @"title": @"B",           @"value": @(GBB)},
                 @{@"type": typeRadio, @"getter": getter, @"setter": setter, @"title": @"Select",      @"value": @(GBSelect)},
                 @{@"type": typeRadio, @"getter": getter, @"setter": setter, @"title": @"Start",       @"value": @(GBStart)},
+                @{@"type": typeRadio, @"getter": getter, @"setter": setter, @"title": @"Rapid A",     @"value": @(GBRapidA)},
+                @{@"type": typeRadio, @"getter": getter, @"setter": setter, @"title": @"Rapid B",     @"value": @(GBRapidB)},
                 @{@"type": typeRadio, @"getter": getter, @"setter": setter, @"title": @"Turbo",       @"value": @(GBTurbo)},
                 @{@"type": typeRadio, @"getter": getter, @"setter": setter, @"title": @"Rewind",      @"value": @(GBRewind)},
                 @{@"type": typeRadio, @"getter": getter, @"setter": setter, @"title": @"Slow-motion", @"value": @(GBUnderclock)},
@@ -709,14 +755,14 @@ static id ValueForItem(NSDictionary *item)
 - (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
 {
     NSDictionary *item = [self itemForIndexPath:indexPath];
-
     
     UITableViewCell *cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleValue1 reuseIdentifier:nil];
     cell.textLabel.text = item[@"title"];
-    if (item[@"type"] == typeSubmenu || item[@"type"] == typeOptionSubmenu || item[@"type"] == typeBlock) {
+    NSString *type = item[@"type"];
+    if (type == typeSubmenu || type == typeOptionSubmenu || type == typeBlock) {
         cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
         cell.selectionStyle = UITableViewCellSelectionStyleBlue;
-        if (item[@"type"] == typeOptionSubmenu) {
+        if (type == typeOptionSubmenu) {
             for (NSDictionary *section in item[@"submenu"]) {
                 for (NSDictionary *item in section[@"items"]) {
                     if (item[@"value"] && [ValueForItem(item) isEqual:item[@"value"]]) {
@@ -730,14 +776,14 @@ static id ValueForItem(NSDictionary *item)
             cell.detailTextLabel.text = [[NSUserDefaults standardUserDefaults] stringForKey:item[@"pref"]];
         }
     }
-    else if (item[@"type"] == typeRadio) {
+    else if (type == typeRadio) {
         id settingValue = ValueForItem(item);
         id itemValue = item[@"value"];
         if (settingValue == itemValue || [settingValue isEqual:itemValue]) {
             cell.accessoryType = UITableViewCellAccessoryCheckmark;
         }
     }
-    else if (item[@"type"] == typeCheck) {
+    else if (type == typeCheck) {
         UISwitch *button = [[UISwitch alloc] init];
         cell.accessoryView = button;
         if ([[NSUserDefaults standardUserDefaults] boolForKey:item[@"pref"]]) {
@@ -747,6 +793,9 @@ static id ValueForItem(NSDictionary *item)
         __weak typeof(self) weakSelf = self;
         id block = ^(){
             [[NSUserDefaults standardUserDefaults] setBool:button.on forKey:item[@"pref"]];
+            if (item[@"block"]) {
+                ((void(^)(GBSettingsViewController *))item[@"block"])(self);
+            }
             unsigned section = [indexPath indexAtPosition:0];
             UITableViewHeaderFooterView *view = [weakSelf.tableView footerViewForSection:section];
             view.textLabel.text = [weakSelf tableView:weakSelf.tableView titleForFooterInSection:section];
@@ -761,7 +810,7 @@ static id ValueForItem(NSDictionary *item)
         [button addTarget:block action:@selector(invoke) forControlEvents:UIControlEventValueChanged];
         cell.selectionStyle = UITableViewCellSelectionStyleNone;
     }
-    else if (item[@"type"] == typeDisabled) {
+    else if (type == typeDisabled) {
         cell.selectionStyle = UITableViewCellSelectionStyleNone;
         if (@available(iOS 13.0, *)) {
             cell.textLabel.textColor = [UIColor separatorColor];
@@ -770,24 +819,34 @@ static id ValueForItem(NSDictionary *item)
             cell.textLabel.textColor = [UIColor colorWithWhite:0 alpha:0.75];
         }
     }
-    else if (item[@"type"] == typeSeparator) {
+    else if (type == typeSeparator) {
         cell.backgroundColor = [UIColor clearColor];
         cell.separatorInset = UIEdgeInsetsZero;
     }
-    else if (item[@"type"] == typeSlider ||
-             item[@"type"] == typeLightTemp) {
+    else if (type == typeSlider ||
+             type == typeLightTemp ||
+             type == typeTurboSlider) {
         CGRect rect = cell.contentView.bounds;
         rect.size.width -= 24;
         rect.size.height -= 24;
         rect.origin.x += 12;
         rect.origin.y += 12;
-        UISlider *slider = [item[@"type"] == typeLightTemp? [GBSlider alloc] : [UISlider alloc] initWithFrame:rect];
+        UISlider *slider = [type == typeLightTemp? [GBSlider alloc] : [UISlider alloc] initWithFrame:rect];
         slider.continuous = true;
         slider.minimumValue = [item[@"min"] floatValue];
         slider.maximumValue = [item[@"max"] floatValue];
         slider.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
         [cell.contentView addSubview:slider];
-        slider.value = [[NSUserDefaults standardUserDefaults] floatForKey:item[@"pref"]];
+        if (type == typeTurboSlider) {
+            slider.value = [[NSUserDefaults standardUserDefaults] floatForKey:item[@"pref"]] ?: 2.0;
+            _turboSlider = slider;
+            if (![[NSUserDefaults standardUserDefaults] floatForKey:item[@"pref"]]) {
+                slider.enabled = false;
+            }
+        }
+        else {
+            slider.value = [[NSUserDefaults standardUserDefaults] floatForKey:item[@"pref"]];
+        }
         cell.selectionStyle = UITableViewCellSelectionStyleNone;
         
         if (@available(iOS 13.0, *)) {
@@ -798,8 +857,19 @@ static id ValueForItem(NSDictionary *item)
             }
         }
         
+        __weak typeof(self) weakSelf = self;
         id block = ^(){
             [[NSUserDefaults standardUserDefaults] setDouble:slider.value forKey:item[@"pref"]];
+            if (type == typeTurboSlider) {
+                unsigned section = [indexPath indexAtPosition:0];
+                UITableViewHeaderFooterView *view = [weakSelf.tableView footerViewForSection:section];
+                view.textLabel.text = [weakSelf tableView:weakSelf.tableView titleForFooterInSection:section];
+                [UIView setAnimationsEnabled:false];
+                [weakSelf.tableView beginUpdates];
+                [view sizeToFit];
+                [weakSelf.tableView endUpdates];
+                [UIView setAnimationsEnabled:true];
+            }
         };
         objc_setAssociatedObject(cell, "RetainedBlock", block, OBJC_ASSOCIATION_RETAIN);
 
@@ -813,13 +883,21 @@ static id ValueForItem(NSDictionary *item)
         cell.separatorInset = UIEdgeInsetsZero;
     }
     cell.imageView.image = item[@"image"];
+    if (@available(iOS 19.0, *)) {
+        if (_iPadRoot) {
+            cell.textLabel.textColor = [UIColor colorWithDynamicProvider:^UIColor *(UITraitCollection *traitCollection) {
+                return cell.isSelected? [UIColor whiteColor] : [UIColor labelColor];
+            }];
+        }
+    }
     return cell;
 }
 
 - (NSIndexPath *)tableView:(UITableView *)tableView willSelectRowAtIndexPath:(NSIndexPath *)indexPath
 {
     NSDictionary *item = [self itemForIndexPath:indexPath];
-    if (item[@"type"] == typeSubmenu || item[@"type"] == typeOptionSubmenu) {
+    NSString *type = item[@"type"];
+    if (type == typeSubmenu || type == typeOptionSubmenu) {
         UITableViewStyle style = UITableViewStyleGrouped;
         if (@available(iOS 13.0, *)) {
             style = UITableViewStyleInsetGrouped;
@@ -843,7 +921,7 @@ static id ValueForItem(NSDictionary *item)
         }
         return indexPath;
     }
-    else if (item[@"type"] == typeRadio) {
+    else if (type == typeRadio) {
         if (item[@"setter"]) {
             ((void(^)(id))item[@"setter"])(item[@"value"]);
         }
@@ -852,7 +930,7 @@ static id ValueForItem(NSDictionary *item)
         }
         [self.tableView reloadData];
     }
-    else if (item[@"type"] == typeBlock) {
+    else if (type == typeBlock) {
         if (((bool(^)(GBSettingsViewController *))item[@"block"])(self)) {
             return indexPath;
         }
@@ -863,11 +941,13 @@ static id ValueForItem(NSDictionary *item)
 - (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
 {
     NSDictionary *item = [self itemForIndexPath:indexPath];
-    if (item[@"type"] == typeSeparator) {
+    NSString *type = item[@"type"];
+    if (type == typeSeparator) {
         return 8;
     }
-    if (item[@"type"] == typeSlider ||
-        item[@"type"] == typeLightTemp) {
+    if (type == typeSlider ||
+        type == typeLightTemp ||
+        type == typeTurboSlider) {
         return 63;
     }
     return [super tableView:tableView heightForRowAtIndexPath:indexPath];
